## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
##  JUnitCapture by Jeckel.
##

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup

import CvConfigParser
import CvPath


# globals
gc = CyGlobalContext()


# ################################################################################
# ##                                                                            ##
# ## This config.ini file has the options allowing you to control the capturing ##
# ## of units.                                                                  ##
# ##                                                                            ##
# ################################################################################


#[JUnitsCapture Mod]


# This option allows you to turn JUnitCapture Mod on or off.
# True - JUnitCapture Mod is on.
# False - JUnitCapture Mod is off.
# Default = True
g_bJUnitCaptureFeaturesEnabled = True


# When a settler unit is attack by a military unit, should it have a chance of being captured.
# Ture - Settlers can be captured.
# False - Settler units can NOT be captured.
# Default = True
g_bSettlersCanBeCaptured = True


# If 'Settlers Can Be Captured = True'
# What percent chance do settlers have to be captured.
# Higher value is greater chance of settlers being captured.
# Lower value is less chance of settler capture.
# Defualt = 95
g_bCaptureSettlerBaseChance = 95


# When a worker unit is attack by a military unit, should it have a chance of being captured.
# Note: In unmodded Civ4 workers are already captured when attacked. If this is set True
#         and the capture tag of the worker is not changed in the xml file, then you will
#         get two workers.
# Ture - Worker units can be captured.
# False - Worker units can NOT be captured.
# Default = True
g_bWorkersCanBeCaptured = True


# If 'Workers Can Be Captured = True'
# What percent chance do workers have to be captured.
# Higher value is greater chance of worker captur.
# Lower value is less chance of worker capture.
# Defualt = 99
g_bCaptureWorkerBaseChance = 99


# When a great person (ie Great Artist, Great Prophet, Great Doctor, ect) is attack by
# a military unit, should it have a chance of being captured.
# Ture - Great People can be captured.
# False - Great People can NOT be captured.
# Default = True
g_bGreatPeopleCanBeCaptured = True


# If 'Great People Can Be Captured = True'
# What percent chance do great peopple have to be captured.
# Higher value is greater chance to capture great people.
# Lower value is less chance of capturing great people.
# Defualt = 75
g_bCaptureGreatPeopleBaseChance = 75


# Is the captured unit changed into a Slave unit before the capturing player gets it.
# True - Capturing player gets a slave unit instead of settler/worker/Great Person.
# False - Capturing player gets the settler/worker/Great Person NOT a slave unit.
# Default = False
g_bCapturedUnitBecomesSlave = False


# ###########################################################################################


# These options let you control what unit stats are kept when a unit is captured.
# All the options bellow only apply when a unit has successfully been captured.


# Should the captured unit lose all its movement when captured.
# True - Captured unit loses all movement.
# False - Captured unit does NOT lose its movement.
# Default = True
g_bCapturedUnitLosesMovement =  True


# Should the Captured Unit have a percent of its health removed.
# The percent of health removed is set in the next option.
# True - A percentage of the captured unit's health is removed.
# False - None of the captured unit's health is removed.
# Default = False
g_bCapturedUnitLosesHealth = False


# If 'Captured Unit Loses Health = True'
# What percentage of Health does the captured unit lose.
# Valid values are 0 to 99 all other values will be treated as 0.
# Default = 50
g_iCapturedUnitPercentHealthLose = 50


# Should the captured unit keep its level and experience.
# True - Captured unit keeps level and exp.
# False - Unit does NOT keep level or experience.
# Default = True
g_bCapturedUnitKeepsLevelAndEXP = True


# Should the captured unit keep ALL its promotions.
# True - Unit keeps ALL its promotions.
# False - Captured unit loses all promotions.
# Default = True
g_bCapturedUnitKeepsAllPromos = True


# Should the captured unit keep its Name.
# True - Unit keeps its name.
# False - Unit loses any unique name.
# Default = True
g_bCapturedUnitKeepsName = True



def loadBasicConfigurationData():
	global g_bSettlersCanBeCaptured
	global g_bCaptureSettlerBaseChance
	global g_bWorkersCanBeCaptured
	global g_bCaptureWorkerBaseChance
	global g_bGreatPeopleCanBeCaptured
	global g_bCaptureGreatPeopleBaseChance
        global g_bCapturedUnitBecomesSlave

        # Load the JUnitCapture Mod Config INI file containing all of the configuration information
	config = CvConfigParser.CvConfigParser("JUnitCapture Mod Config.ini")
		
	# If the JUnitCapture Mod's config.ini file was found then load its values
	if(config != None):
                g_bSettlersCanBeCaptured = config.getboolean("JUnitCapture Mod", "Settlers Can Be Captured", True)
                g_bCaptureSettlerBaseChance = config.getint("JUnitCapture Mod", "Capture Settler Base Chance", 95)
                g_bWorkersCanBeCaptured = config.getboolean("JUnitCapture Mod", "Workers Can Be Captured", True)
                g_bCaptureWorkerBaseChance = config.getint("JUnitCapture Mod", "Capture Worker Base Chance", 99)
                g_bGreatPeopleCanBeCaptured = config.getboolean("JUnitCapture Mod", "Great People Can Be Captured", True)
                g_bCaptureGreatPeopleBaseChance = config.getint("JUnitCapture Mod", "Capture Great People Base Chance", 75)
                g_bCapturedUnitBecomesSlave = config.getboolean("JUnitCapture Mod", "Captured Unit Becomes Slave", False)



def loadCreationConfigurationData():
	global g_bCapturedUnitLosesMovement
	global g_bCapturedUnitLosesHealth
	global g_iCapturedUnitPercentHealthLose
	global g_bCapturedUnitKeepsLevelAndEXP
	global g_bCapturedUnitKeepsAllPromos
	global g_bCapturedUnitKeepsName

        # Load the JUnitCapture Mod Config INI file containing all of the configuration information
	config = CvConfigParser.CvConfigParser("JUnitCapture Mod Config.ini")
		
	# If the JUnitCapture Mod's config.ini file was found then load its values
	if(config != None):
                g_bCapturedUnitLosesMovement = config.getboolean("JUnitCapture Mod", "Captured Unit Loses Movement", True)
                g_bapturedUnitLosesHealth = config.getboolean("JUnitCapture Mod", "Captured Unit Loses Health", False)
                g_iCapturedUnitPercentHealthLose = config.getint("JUnitCapture Mod", "Captured Unit Percent Health Lose", 50)
                if (g_iCapturedUnitPercentHealthLose < 0) or (g_iCapturedUnitPercentHealthLose > 99):
                        g_iCapturedUnitPercentHealthLose = 0
                g_bCapturedUnitKeepsLevelAndEXP = config.getboolean("JUnitCapture Mod", "Captured Unit Keeps Level And EXP", True)
                g_bCapturedUnitKeepsAllPromos = config.getboolean("JUnitCapture Mod", "Captured Unit Keeps All Promos", True)
                g_bCapturedUnitKeepsName = config.getboolean("JUnitCapture Mod", "Captured Unit Keeps Name", True)



class JUnitCapture:
        def __init__(self):
		self.city = None

		#loadConfigurationData()


        ###################################################################
        # These methods produce messages in the log and/or on the screen. #
        ###################################################################

        def alertActivePlayer(self, message, iColor = 7):
                ePlayer = gc.getGame().getActivePlayer()
                eventMessageTimeLong = gc.getDefineINT("EVENT_MESSAGE_TIME_LONG")
                szIcon = None
                iFlashX = 0
                iFlashY = 0

                szString = str(message)

                CyInterface().addMessage(ePlayer, True, eventMessageTimeLong,
                                         szString, None, 0, szIcon, ColorTypes(iColor),
                                         iFlashX, iFlashY, True, True)

        def alertIconPlayer(self, message, pUnit, szIcon = None, iColor = 7):
                ePlayer = gc.getGame().getActivePlayer()
                eventMessageTimeLong = gc.getDefineINT("EVENT_MESSAGE_TIME_LONG")

                strMessage = str(message)

                CyInterface().addMessage(ePlayer, True, eventMessageTimeLong,
                                         strMessage, None, 0, szIcon, ColorTypes(iColor),
                                         pUnit.getX(), pUnit.getY(), True, True)

        ##########################################################################################
        # These are the is methods. They take some form of input and return bool True or False.  #
        ##########################################################################################


        def isUnitCaptured(self, pUnit, iRandNum):

                # Return immediately if the unit passed in is invalid
		if (pUnit == None):
			return False
			
		# Return immediately if the unit passed in is invalid
		if (pUnit.isNone()):
			return False

		# If iRandNum is less then or equal to 0 return False
		if (iRandNum <= 0):
                        return False

                loadBasicConfigurationData()

                unitClass = pUnit.getUnitClassType()

                # Set the lists of unitclasses that can be captured
                listSettlerClass = ["UNITCLASS_SETTLER"]
                listWorkerClass = ["UNITCLASS_WORKER", "UNITCLASS_SLAVE"]
		listGreatPeopleClass = ["UNITCLASS_PROPHET","UNITCLASS_ARTIST",
                                        "UNITCLASS_SCIENTIST","UNITCLASS_MERCHANT",
                                        "UNITCLASS_ENGINEER","UNITCLASS_GENERAL",
                                        "UNITCLASS_STATESMAN","UNITCLASS_DOCTOR"]
		listMiscClass = []

                # Check if pUnit is a settler unit
		for xUnitClass in listSettlerClass:
                        if (unitClass == gc.getInfoTypeForString(xUnitClass)) and (g_bSettlersCanBeCaptured) and (g_bCaptureSettlerBaseChance >= iRandNum):
                                return True

                # Check if pUnit is a worker unit
                for xUnitClass in listWorkerClass:
                        if (unitClass == gc.getInfoTypeForString(xUnitClass)) and (g_bWorkersCanBeCaptured) and (g_bCaptureWorkerBaseChance >= iRandNum):
                                return True

                # Check if pUnit is a Great Person unit
                for xUnitClass in listGreatPeopleClass:
                        if (unitClass == gc.getInfoTypeForString(xUnitClass)) and (g_bGreatPeopleCanBeCaptured) and (g_bCaptureGreatPeopleBaseChance >= iRandNum):
                                return True

                # Check if pUnit is a misc capturable unit
                for xUnitClass in listMiscClass:
                        if (unitClass == gc.getInfoTypeForString(xUnitClass)):
                                return True

                return False


        ########################################################
        # These are the run methods and don't return anything. #
        ########################################################

        def runFinishUnit(self, pNewUnit, pLoserUnit):

                # Return immediately if the unit passed in is invalid
		if (pNewUnit == None):
			return None
			
		# Return immediately if the unit passed in is invalid
		if (pNewUnit.isNone()):
			return None

                # Return immediately if the unit passed in is invalid
		if (pLoserUnit == None):
			return None
			
		# Return immediately if the unit passed in is invalid
		if (pLoserUnit.isNone()):
			return None

		loadCreationConfigurationData()

		# Finish setting unit's movement
		if (g_bCapturedUnitLosesMovement):
                        pNewUnit.finishMoves()

                # Finish setting unit's damage
                if (g_bCapturedUnitLosesHealth):
                        pNewUnit.setDamage(g_iCapturedUnitPercentHealthLose, False)

                # Finish transferring unit's level and experience
                if (g_bCapturedUnitKeepsLevelAndEXP):
                        pNewUnit.setExperience(pLoserUnit.getExperience(), -1)
                        pNewUnit.setLevel(pLoserUnit.getLevel())

                # Finish transferring unit's name
                if (g_bCapturedUnitKeepsName):
                        pNewUnit.setName(pLoserUnit.getNameNoDesc())

                # Finish transferring unit's promotions
                if (g_bCapturedUnitKeepsAllPromos):
                        # Go through all of the promotions in the game	
                        for iPromotion in range(gc.getNumPromotionInfos()):
                                # If pLoserUnit has the promotion, give it to the pNewUnit
                                if (pLoserUnit.isHasPromotion(iPromotion)):
                                        pNewUnit.setHasPromotion(iPromotion, True)


        ## This Function is not used. ##
        ## It is part of the next version of the mod and doesn't function yet. ##
        def runCaptureStack(self, pWinnerUnit, pLoserUnit):

                pWinnerPlayer = gc.getPlayer(pWinnerUnit.getOwner())
		pLoserPlayer = gc.getPlayer(pLoserUnit.getOwner())

		pLoserPlot = pLoserUnit.plot()

		

		for iUnit in range(pLoserPlot.getNumUnits()):
                        pUnit = pLoserPlot.getUnit(iUnit)

                        if (pUnit.getOwner() == pLoserPlayer):
                                # Roll the random number to is if unit is captured
                                randCaptureRoll = CyGame().getSorenRandNum(101, "Random Capture Unit Roll")

                                # Check if the unit can be captured and if it is captured, if not return None
                                if (self.isUnitCaptured(pLoserUnit, randCaptureRoll)):

                                        # Create the new instance of slave unit	
                                        pNewUnit = pWinnerPlayer.initUnit(pLoserUnit.getUnitType(),  pWinnerUnit.getX(), pWinnerUnit.getY(), UnitAITypes.NO_UNITAI)
		
                                        #
                                        self.runFinishUnit(pNewUnit, pLoserUnit)


        ##########################################
        # These are the onEvent Handler methods. #
        ##########################################

	def onCombatResult(self, argsList):
		'Combat Result'

		pWinnerUnit, pLoserUnit = argsList

		pWinnerPlayer = gc.getPlayer(pWinnerUnit.getOwner())

                # Roll the random number to is if unit is captured
		randCaptureRoll = CyGame().getSorenRandNum(101, "Random Capture Unit Roll")

                listSlaveUnits = ["UNIT_SLAVE", "UNIT_AFRICAN_SLAVE"]

		# Get a random slave type
		iRandSlave = CyGame().getSorenRandNum(len(listSlaveUnits), "Random Captured Slave")
		strRandSlave = listSlaveUnits[iRandSlave]

                # Check if the unit can be captured and if it is captured, if not return None
		if (self.isUnitCaptured(pLoserUnit, randCaptureRoll)):

                        loadBasicConfigurationData()

                        # Check if new unit should made into a slave
                        if (g_bCapturedUnitBecomesSlave):#gc.getInfoTypeForString(xUnitClass)

                                # Create the new instance of a slave unit
                                pNewUnit = pWinnerPlayer.initUnit(gc.getInfoTypeForString(strRandSlave), pWinnerUnit.getX(), pWinnerUnit.getY(), UnitAITypes.NO_UNITAI)

                        else:
                                # Create the new instance of the unit	
                                pNewUnit = pWinnerPlayer.initUnit(pLoserUnit.getUnitType(), pWinnerUnit.getX(), pWinnerUnit.getY(), UnitAITypes.NO_UNITAI)
		
                        #
                        self.runFinishUnit(pNewUnit, pLoserUnit)
































	#def onUnitMove(self, argsList):
		#'unit move'
		#pPlot, pUnit = argsList

		#pPlayer = gc.getPlayer(pUnit.getOwner())

                #strUnitID = str(pUnit.getOwner()) + "-" + str(pUnit.getID())

                #iStateReligion = pPlayer.getStateReligion()

                #typeStateReligion = gc.getReligionInfo(iStateReligion).getType()

                #self.alertActivePlayer(iStateReligion)
                #self.alertActivePlayer(typeStateReligion)



















